"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetTypeManifestVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const visitors_1 = require("../../visitors");
const fragments_1 = require("./fragments");
const ImportMap_1 = require("./ImportMap");
const TypeManifest_1 = require("./TypeManifest");
function getEncoderFunction(name) {
    return `get${(0, shared_1.pascalCase)(name)}Encoder`;
}
function getDecoderFunction(name) {
    return `get${(0, shared_1.pascalCase)(name)}Decoder`;
}
class GetTypeManifestVisitor {
    constructor() {
        this.parentName = null;
    }
    visitRoot() {
        throw new Error('Cannot get type manifest for root node. Please select a child node.');
    }
    visitProgram() {
        throw new Error('Cannot get type manifest for program node. Please select a child node.');
    }
    visitAccount(account) {
        return (0, visitors_1.visit)(account.data, this);
    }
    visitAccountData(accountData) {
        this.parentName = {
            strict: (0, shared_1.pascalCase)(accountData.name),
            loose: `${(0, shared_1.pascalCase)(accountData.name)}Args`,
        };
        const manifest = accountData.link
            ? (0, visitors_1.visit)(accountData.link, this)
            : (0, visitors_1.visit)(accountData.struct, this);
        this.parentName = null;
        return manifest;
    }
    visitInstruction(instruction) {
        return (0, visitors_1.visit)(instruction.dataArgs, this);
    }
    visitInstructionAccount() {
        throw new Error('Cannot get type manifest for instruction account node. Please select a another node.');
    }
    visitInstructionDataArgs(instructionDataArgs) {
        this.parentName = {
            strict: (0, shared_1.pascalCase)(instructionDataArgs.name),
            loose: `${(0, shared_1.pascalCase)(instructionDataArgs.name)}Args`,
        };
        const manifest = instructionDataArgs.link
            ? (0, visitors_1.visit)(instructionDataArgs.link, this)
            : (0, visitors_1.visit)(instructionDataArgs.struct, this);
        this.parentName = null;
        return manifest;
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        this.parentName = {
            strict: (0, shared_1.pascalCase)(instructionExtraArgs.name),
            loose: `${(0, shared_1.pascalCase)(instructionExtraArgs.name)}Args`,
        };
        const manifest = instructionExtraArgs.link
            ? (0, visitors_1.visit)(instructionExtraArgs.link, this)
            : (0, visitors_1.visit)(instructionExtraArgs.struct, this);
        this.parentName = null;
        return manifest;
    }
    visitDefinedType(definedType) {
        this.parentName = {
            strict: (0, shared_1.pascalCase)(definedType.name),
            loose: `${(0, shared_1.pascalCase)(definedType.name)}Args`,
        };
        const manifest = (0, visitors_1.visit)(definedType.data, this);
        this.parentName = null;
        return manifest;
    }
    visitError() {
        throw new Error('Cannot get type manifest for error node.');
    }
    visitArrayType(arrayType) {
        const childManifest = (0, visitors_1.visit)(arrayType.child, this);
        childManifest.looseType.mapRender((r) => `Array<${r}>`);
        childManifest.strictType.mapRender((r) => `Array<${r}>`);
        const sizeManifest = this.getArrayLikeSizeOption(arrayType.size);
        const encoderOptions = sizeManifest.encoder.render
            ? `, { ${sizeManifest.encoder.render} }`
            : '';
        const decoderOptions = sizeManifest.decoder.render
            ? `, { ${sizeManifest.decoder.render} }`
            : '';
        childManifest.encoder
            .mapRender((r) => `getArrayEncoder(${r + encoderOptions})`)
            .mergeImportsWith(sizeManifest.encoder)
            .addImports('solanaCodecsDataStructures', 'getArrayEncoder');
        childManifest.decoder
            .mapRender((r) => `getArrayDecoder(${r + decoderOptions})`)
            .mergeImportsWith(sizeManifest.decoder)
            .addImports('solanaCodecsDataStructures', 'getArrayDecoder');
        return childManifest;
    }
    visitLinkType(linkType) {
        const pascalCaseDefinedType = (0, shared_1.pascalCase)(linkType.name);
        const encoderFunction = getEncoderFunction(linkType.name);
        const decoderFunction = getDecoderFunction(linkType.name);
        const importFrom = linkType.importFrom === 'generated'
            ? 'generatedTypes'
            : linkType.importFrom;
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)(pascalCaseDefinedType).addImports(importFrom, pascalCaseDefinedType),
            looseType: (0, fragments_1.fragment)(`${pascalCaseDefinedType}Args`).addImports(importFrom, `${pascalCaseDefinedType}Args`),
            encoder: (0, fragments_1.fragment)(`${encoderFunction}()`).addImports(importFrom, encoderFunction),
            decoder: (0, fragments_1.fragment)(`${decoderFunction}()`).addImports(importFrom, decoderFunction),
        };
    }
    visitEnumType(enumType) {
        const { parentName } = this;
        this.parentName = null;
        const variantNames = enumType.variants.map((v) => (0, shared_1.pascalCase)(v.name));
        const encoderImports = new ImportMap_1.ImportMap();
        const decoderImports = new ImportMap_1.ImportMap();
        const encoderOptions = [];
        const decoderOptions = [];
        if (enumType.size.format !== 'u8' || enumType.size.endian !== 'le') {
            const sizeManifest = (0, visitors_1.visit)(enumType.size, this);
            encoderImports.mergeWith(sizeManifest.encoder);
            decoderImports.mergeWith(sizeManifest.decoder);
            encoderOptions.push(`size: ${sizeManifest.encoder.render}`);
            decoderOptions.push(`size: ${sizeManifest.decoder.render}`);
        }
        if (parentName === null || parentName === void 0 ? void 0 : parentName.strict) {
            encoderOptions.push(`description: '${(0, shared_1.pascalCase)(parentName.strict)}'`);
            decoderOptions.push(`description: '${(0, shared_1.pascalCase)(parentName.strict)}'`);
        }
        const encoderOptionsAsString = encoderOptions.length > 0 ? `, { ${encoderOptions.join(', ')} }` : '';
        const decoderOptionsAsString = decoderOptions.length > 0 ? `, { ${decoderOptions.join(', ')} }` : '';
        if (nodes.isScalarEnum(enumType)) {
            if (parentName === null) {
                throw new Error('Scalar enums cannot be inlined and must be introduced ' +
                    'via a defined type. Ensure you are not inlining a ' +
                    'defined type that is a scalar enum through a visitor.');
            }
            return {
                isEnum: true,
                strictType: (0, fragments_1.fragment)(`{ ${variantNames.join(', ')} }`),
                looseType: (0, fragments_1.fragment)(`{ ${variantNames.join(', ')} }`),
                encoder: (0, fragments_1.fragment)(`getScalarEnumEncoder<${parentName.strict}>(${parentName.strict + encoderOptionsAsString})`, encoderImports.add('solanaCodecsDataStructures', 'getScalarEnumEncoder')),
                decoder: (0, fragments_1.fragment)(`getScalarEnumDecoder<${parentName.strict}>(${parentName.strict + decoderOptionsAsString})`, decoderImports.add('solanaCodecsDataStructures', 'getScalarEnumDecoder')),
            };
        }
        const variants = enumType.variants.map((variant) => {
            const variantName = (0, shared_1.pascalCase)(variant.name);
            this.parentName = parentName
                ? {
                    strict: `GetDataEnumKindContent<${parentName.strict}, '${variantName}'>`,
                    loose: `GetDataEnumKindContent<${parentName.loose}, '${variantName}'>`,
                }
                : null;
            const variantManifest = (0, visitors_1.visit)(variant, this);
            this.parentName = null;
            return variantManifest;
        });
        const mergedManifest = (0, TypeManifest_1.mergeManifests)(variants, (renders) => renders.join(' | '), (renders) => renders.join(', '));
        const typeParam = parentName ? parentName.strict : 'any';
        mergedManifest.encoder
            .mapRender((r) => `getDataEnumEncoder<${typeParam}>([${r}]${encoderOptionsAsString})`)
            .addImports('solanaCodecsDataStructures', [
            'GetDataEnumKindContent',
            'getDataEnumEncoder',
        ]);
        mergedManifest.decoder
            .mapRender((r) => `getDataEnumDecoder<${typeParam}>([${r}]${decoderOptionsAsString})`)
            .addImports('solanaCodecsDataStructures', [
            'GetDataEnumKind',
            'getDataEnumDecoder',
        ]);
        return mergedManifest;
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        const name = (0, shared_1.pascalCase)(enumEmptyVariantType.name);
        const kindAttribute = `__kind: "${name}"`;
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)(`{ ${kindAttribute} }`),
            looseType: (0, fragments_1.fragment)(`{ ${kindAttribute} }`),
            encoder: (0, fragments_1.fragment)(`['${name}', getUnitEncoder()]`).addImports('solanaCodecsDataStructures', 'getUnitEncoder'),
            decoder: (0, fragments_1.fragment)(`['${name}', getUnitDecoder()]`).addImports('solanaCodecsDataStructures', 'getUnitDecoder'),
        };
    }
    visitEnumStructVariantType(enumStructVariantType) {
        const name = (0, shared_1.pascalCase)(enumStructVariantType.name);
        const kindAttribute = `__kind: "${name}"`;
        const structManifest = (0, visitors_1.visit)(enumStructVariantType.struct, this);
        structManifest.strictType.mapRender((r) => `{ ${kindAttribute},${r.slice(1, -1)}}`);
        structManifest.looseType.mapRender((r) => `{ ${kindAttribute},${r.slice(1, -1)}}`);
        structManifest.encoder.mapRender((r) => `['${name}', ${r}]`);
        structManifest.decoder.mapRender((r) => `['${name}', ${r}]`);
        return structManifest;
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        const name = (0, shared_1.pascalCase)(enumTupleVariantType.name);
        const kindAttribute = `__kind: "${name}"`;
        const struct = nodes.structTypeNode([
            nodes.structFieldTypeNode({
                name: 'fields',
                child: enumTupleVariantType.tuple,
            }),
        ]);
        const structManifest = (0, visitors_1.visit)(struct, this);
        structManifest.strictType.mapRender((r) => `{ ${kindAttribute},${r.slice(1, -1)}}`);
        structManifest.looseType.mapRender((r) => `{ ${kindAttribute},${r.slice(1, -1)}}`);
        structManifest.encoder.mapRender((r) => `['${name}', ${r}]`);
        structManifest.decoder.mapRender((r) => `['${name}', ${r}]`);
        return structManifest;
    }
    visitMapType(mapType) {
        const key = (0, visitors_1.visit)(mapType.key, this);
        const value = (0, visitors_1.visit)(mapType.value, this);
        const mergedManifest = (0, TypeManifest_1.mergeManifests)([key, value], ([k, v]) => `Map<${k}, ${v}>`, ([k, v]) => `${k}, ${v}`);
        const sizeManifest = this.getArrayLikeSizeOption(mapType.size);
        const encoderOptions = sizeManifest.encoder.render
            ? `, { ${sizeManifest.encoder.render} }`
            : '';
        const decoderOptions = sizeManifest.decoder.render
            ? `, { ${sizeManifest.decoder.render} }`
            : '';
        mergedManifest.encoder
            .mapRender((r) => `getMapEncoder(${r}${encoderOptions})`)
            .addImports('solanaCodecsDataStructures', 'getMapEncoder');
        mergedManifest.decoder
            .mapRender((r) => `getMapDecoder(${r}${decoderOptions})`)
            .addImports('solanaCodecsDataStructures', 'getMapDecoder');
        return mergedManifest;
    }
    visitOptionType(optionType) {
        const childManifest = (0, visitors_1.visit)(optionType.child, this);
        childManifest.strictType
            .mapRender((r) => `Option<${r}>`)
            .addImports('solanaOptions', 'Option');
        childManifest.looseType
            .mapRender((r) => `OptionOrNullable<${r}>`)
            .addImports('solanaOptions', 'OptionOrNullable');
        const encoderOptions = [];
        const decoderOptions = [];
        // Prefix option.
        if (optionType.prefix.format !== 'u8' ||
            optionType.prefix.endian !== 'le') {
            const prefixManifest = (0, visitors_1.visit)(optionType.prefix, this);
            childManifest.encoder.mergeImportsWith(prefixManifest.encoder);
            childManifest.decoder.mergeImportsWith(prefixManifest.decoder);
            encoderOptions.push(`prefix: ${prefixManifest.encoder.render}`);
            decoderOptions.push(`prefix: ${prefixManifest.decoder.render}`);
        }
        // Fixed option.
        if (optionType.fixed) {
            encoderOptions.push(`fixed: true`);
            decoderOptions.push(`fixed: true`);
        }
        const encoderOptionsAsString = encoderOptions.length > 0 ? `, { ${encoderOptions.join(', ')} }` : '';
        const decoderOptionsAsString = decoderOptions.length > 0 ? `, { ${decoderOptions.join(', ')} }` : '';
        childManifest.encoder
            .mapRender((r) => `getOptionEncoder(${r + encoderOptionsAsString})`)
            .addImports('solanaOptions', 'getOptionEncoder');
        childManifest.decoder
            .mapRender((r) => `getOptionDecoder(${r + decoderOptionsAsString})`)
            .addImports('solanaOptions', 'getOptionDecoder');
        return childManifest;
    }
    visitSetType(setType) {
        const childManifest = (0, visitors_1.visit)(setType.child, this);
        childManifest.strictType.mapRender((r) => `Set<${r}>`);
        childManifest.looseType.mapRender((r) => `Set<${r}>`);
        const sizeManifest = this.getArrayLikeSizeOption(setType.size);
        const encoderOptions = sizeManifest.encoder.render
            ? `, { ${sizeManifest.encoder.render} }`
            : '';
        const decoderOptions = sizeManifest.decoder.render
            ? `, { ${sizeManifest.decoder.render} }`
            : '';
        childManifest.encoder
            .mergeImportsWith(sizeManifest.encoder)
            .mapRender((r) => `getSetEncoder(${r + encoderOptions})`)
            .addImports('solanaCodecsDataStructures', 'getSetEncoder');
        childManifest.decoder
            .mergeImportsWith(sizeManifest.decoder)
            .mapRender((r) => `getSetDecoder(${r + decoderOptions})`)
            .addImports('solanaCodecsDataStructures', 'getSetDecoder');
        return childManifest;
    }
    visitStructType(structType) {
        const { parentName } = this;
        this.parentName = null;
        const structDescription = (parentName === null || parentName === void 0 ? void 0 : parentName.strict) && !parentName.strict.match(/['"<>]/)
            ? `, { description: '${(0, shared_1.pascalCase)(parentName.strict)}' }`
            : '';
        const optionalFields = structType.fields.filter((f) => f.defaultsTo !== null);
        const mergedManifest = (0, TypeManifest_1.mergeManifests)(structType.fields.map((field) => (0, visitors_1.visit)(field, this)), (renders) => `{ ${renders.join('')} }`, (renders) => `([${renders.join(', ')}]${structDescription})`);
        let decoderType = parentName === null || parentName === void 0 ? void 0 : parentName.strict;
        if (!decoderType) {
            decoderType = mergedManifest.strictType.render;
            mergedManifest.decoder.mergeImportsWith(mergedManifest.strictType);
        }
        let encoderType = parentName === null || parentName === void 0 ? void 0 : parentName.loose;
        if (!encoderType || optionalFields.length > 0) {
            const nonDefaultsMergedManifest = (0, TypeManifest_1.mergeManifests)(structType.fields.map((field) => (0, visitors_1.visit)({ ...field, defaultsTo: null }, this)), (renders) => `{ ${renders.join('')} }`, (renders) => `([${renders.join(', ')}]${structDescription})`);
            encoderType = nonDefaultsMergedManifest.looseType.render;
            mergedManifest.encoder.mergeImportsWith(nonDefaultsMergedManifest.looseType);
        }
        mergedManifest.encoder
            .mapRender((r) => `getStructEncoder<${encoderType}>${r}`)
            .addImports('solanaCodecsDataStructures', 'getStructEncoder');
        mergedManifest.decoder
            .mapRender((r) => `getStructDecoder<${decoderType}>${r}`)
            .addImports('solanaCodecsDataStructures', 'getStructDecoder');
        if (optionalFields.length === 0) {
            return mergedManifest;
        }
        const defaultValues = optionalFields
            .map((f) => {
            const key = (0, shared_1.camelCase)(f.name);
            const defaultsTo = f.defaultsTo;
            const { render: renderedValue, imports } = (0, fragments_1.getValueNodeFragment)(defaultsTo.value);
            mergedManifest.encoder.mergeImportsWith(imports);
            return defaultsTo.strategy === 'omitted'
                ? `${key}: ${renderedValue}`
                : `${key}: value.${key} ?? ${renderedValue}`;
        })
            .join(', ');
        mergedManifest.encoder
            .mapRender((r) => `mapEncoder(${r}, (value) => ({ ...value, ${defaultValues} }))`)
            .addImports('solanaCodecsCore', 'mapEncoder');
        return mergedManifest;
    }
    visitStructFieldType(structFieldType) {
        const name = (0, shared_1.camelCase)(structFieldType.name);
        const childManifest = (0, visitors_1.visit)(structFieldType.child, this);
        const docblock = this.createDocblock(structFieldType.docs);
        const originalLooseType = childManifest.looseType.render;
        childManifest.strictType.mapRender((r) => `${docblock}${name}: ${r}; `);
        childManifest.looseType.mapRender((r) => `${docblock}${name}: ${r}; `);
        childManifest.encoder.mapRender((r) => `['${name}', ${r}]`);
        childManifest.decoder.mapRender((r) => `['${name}', ${r}]`);
        // No default value.
        if (structFieldType.defaultsTo === null) {
            return childManifest;
        }
        // Optional default value.
        if (structFieldType.defaultsTo.strategy === 'optional') {
            childManifest.looseType.setRender(`${docblock}${name}?: ${originalLooseType}; `);
            return childManifest;
        }
        // Omitted default value.
        childManifest.looseType = (0, fragments_1.fragment)('');
        return childManifest;
    }
    visitTupleType(tupleType) {
        const children = tupleType.children.map((item) => (0, visitors_1.visit)(item, this));
        const mergedManifest = (0, TypeManifest_1.mergeManifests)(children, (types) => `[${types.join(', ')}]`, (codecs) => `[${codecs.join(', ')}]`);
        mergedManifest.encoder
            .mapRender((render) => `getTupleEncoder(${render})`)
            .addImports('solanaCodecsDataStructures', 'getTupleEncoder');
        mergedManifest.decoder
            .mapRender((render) => `getTupleDecoder(${render})`)
            .addImports('solanaCodecsDataStructures', 'getTupleDecoder');
        return mergedManifest;
    }
    visitBoolType(boolType) {
        const encoderImports = new ImportMap_1.ImportMap().add('solanaCodecsDataStructures', 'getBooleanEncoder');
        const decoderImports = new ImportMap_1.ImportMap().add('solanaCodecsDataStructures', 'getBooleanDecoder');
        let sizeEncoder = '';
        let sizeDecoder = '';
        if (boolType.size.format !== 'u8' || boolType.size.endian !== 'le') {
            const size = (0, visitors_1.visit)(boolType.size, this);
            encoderImports.mergeWith(size.encoder);
            decoderImports.mergeWith(size.decoder);
            sizeEncoder = `{ size: ${size.encoder.render} }`;
            sizeDecoder = `{ size: ${size.decoder.render} }`;
        }
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)('boolean'),
            looseType: (0, fragments_1.fragment)('boolean'),
            encoder: (0, fragments_1.fragment)(`getBooleanEncoder(${sizeEncoder})`, encoderImports),
            decoder: (0, fragments_1.fragment)(`getBooleanDecoder(${sizeDecoder})`, decoderImports),
        };
    }
    visitBytesType(bytesType) {
        const encoderImports = new ImportMap_1.ImportMap().add('solanaCodecsDataStructures', 'getBytesEncoder');
        const decoderImports = new ImportMap_1.ImportMap().add('solanaCodecsDataStructures', 'getBytesDecoder');
        const encoderOptions = [];
        const decoderOptions = [];
        // Size option.
        if (bytesType.size.kind === 'prefixed') {
            const prefix = (0, visitors_1.visit)(bytesType.size.prefix, this);
            encoderImports.mergeWith(prefix.encoder);
            decoderImports.mergeWith(prefix.decoder);
            encoderOptions.push(`size: ${prefix.encoder.render}`);
            decoderOptions.push(`size: ${prefix.decoder.render}`);
        }
        else if (bytesType.size.kind === 'fixed') {
            encoderOptions.push(`size: ${bytesType.size.value}`);
            decoderOptions.push(`size: ${bytesType.size.value}`);
        }
        const encoderOptionsAsString = encoderOptions.length > 0 ? `{ ${encoderOptions.join(', ')} }` : '';
        const decoderOptionsAsString = decoderOptions.length > 0 ? `{ ${decoderOptions.join(', ')} }` : '';
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)('Uint8Array'),
            looseType: (0, fragments_1.fragment)('Uint8Array'),
            encoder: (0, fragments_1.fragment)(`getBytesEncoder(${encoderOptionsAsString})`, encoderImports),
            decoder: (0, fragments_1.fragment)(`getBytesDecoder(${decoderOptionsAsString})`, decoderImports),
        };
    }
    visitNumberType(numberType) {
        const encoderFunction = getEncoderFunction(numberType.format);
        const decoderFunction = getDecoderFunction(numberType.format);
        const isBigNumber = ['u64', 'u128', 'i64', 'i128'].includes(numberType.format);
        const encoderImports = new ImportMap_1.ImportMap().add('solanaCodecsNumbers', encoderFunction);
        const decoderImports = new ImportMap_1.ImportMap().add('solanaCodecsNumbers', decoderFunction);
        let endianness = '';
        if (numberType.endian === 'be') {
            encoderImports.add('solanaCodecsNumbers', 'Endian');
            decoderImports.add('solanaCodecsNumbers', 'Endian');
            endianness = '{ endian: Endian.Big }';
        }
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)(isBigNumber ? 'bigint' : 'number'),
            looseType: (0, fragments_1.fragment)(isBigNumber ? 'number | bigint' : 'number'),
            encoder: (0, fragments_1.fragment)(`${encoderFunction}(${endianness})`, encoderImports),
            decoder: (0, fragments_1.fragment)(`${decoderFunction}(${endianness})`, decoderImports),
        };
    }
    visitNumberWrapperType(numberWrapperType) {
        // TODO: Support number wrapper types.
        const { number } = numberWrapperType;
        return (0, visitors_1.visit)(number, this);
    }
    visitPublicKeyType() {
        const imports = new ImportMap_1.ImportMap().add('solanaAddresses', 'Base58EncodedAddress');
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)('Base58EncodedAddress', imports),
            looseType: (0, fragments_1.fragment)('Base58EncodedAddress', imports),
            encoder: (0, fragments_1.fragment)('getAddressEncoder()').addImports('solanaAddresses', 'getAddressEncoder'),
            decoder: (0, fragments_1.fragment)('getAddressDecoder()').addImports('solanaAddresses', 'getAddressDecoder'),
        };
    }
    visitStringType(stringType) {
        const encoderImports = new ImportMap_1.ImportMap().add('solanaCodecsStrings', 'getStringEncoder');
        const decoderImports = new ImportMap_1.ImportMap().add('solanaCodecsStrings', 'getStringDecoder');
        const encoderOptions = [];
        const decoderOptions = [];
        // Encoding option.
        if (stringType.encoding !== 'utf8') {
            const encoderFunction = getEncoderFunction(stringType.encoding);
            const decoderFunction = getDecoderFunction(stringType.encoding);
            encoderImports.add('solanaCodecsStrings', encoderFunction);
            decoderImports.add('solanaCodecsStrings', decoderFunction);
            encoderOptions.push(`encoding: ${encoderFunction}`);
            decoderOptions.push(`encoding: ${decoderFunction}`);
        }
        // Size option.
        if (stringType.size.kind === 'remainder') {
            encoderOptions.push(`size: 'variable'`);
            decoderOptions.push(`size: 'variable'`);
        }
        else if (stringType.size.kind === 'fixed') {
            encoderOptions.push(`size: ${stringType.size.value}`);
            decoderOptions.push(`size: ${stringType.size.value}`);
        }
        else if (stringType.size.prefix.format !== 'u32' ||
            stringType.size.prefix.endian !== 'le') {
            const prefix = (0, visitors_1.visit)(stringType.size.prefix, this);
            encoderImports.mergeWith(prefix.encoder.imports);
            decoderImports.mergeWith(prefix.decoder.imports);
            encoderOptions.push(`size: ${prefix.encoder.render}`);
            decoderOptions.push(`size: ${prefix.decoder.render}`);
        }
        const encoderOptionsAsString = encoderOptions.length > 0 ? `{ ${encoderOptions.join(', ')} }` : '';
        const decoderOptionsAsString = decoderOptions.length > 0 ? `{ ${decoderOptions.join(', ')} }` : '';
        return {
            isEnum: false,
            strictType: (0, fragments_1.fragment)('string'),
            looseType: (0, fragments_1.fragment)('string'),
            encoder: (0, fragments_1.fragment)(`getStringEncoder(${encoderOptionsAsString})`, encoderImports),
            decoder: (0, fragments_1.fragment)(`getStringDecoder(${decoderOptionsAsString})`, decoderImports),
        };
    }
    createDocblock(docs) {
        if (docs.length <= 0)
            return '';
        if (docs.length === 1)
            return `\n/** ${docs[0]} */\n`;
        const lines = docs.map((doc) => ` * ${doc}`);
        return `\n/**\n${lines.join('\n')}\n */\n`;
    }
    getArrayLikeSizeOption(size) {
        if (size.kind === 'fixed') {
            return {
                encoder: (0, fragments_1.fragment)(`size: ${size.value}`),
                decoder: (0, fragments_1.fragment)(`size: ${size.value}`),
            };
        }
        if (size.kind === 'remainder') {
            return {
                encoder: (0, fragments_1.fragment)(`size: 'remainder'`),
                decoder: (0, fragments_1.fragment)(`size: 'remainder'`),
            };
        }
        if (size.prefix.format === 'u32' && size.prefix.endian === 'le') {
            return { encoder: (0, fragments_1.fragment)(''), decoder: (0, fragments_1.fragment)('') };
        }
        const prefixManifest = (0, visitors_1.visit)(size.prefix, this);
        prefixManifest.encoder.mapRender((r) => `size: ${r}`);
        prefixManifest.decoder.mapRender((r) => `size: ${r}`);
        return prefixManifest;
    }
}
exports.GetTypeManifestVisitor = GetTypeManifestVisitor;
//# sourceMappingURL=GetTypeManifestVisitor.js.map