"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionInputTypeFragment = void 0;
const shared_1 = require("../../../shared");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
function getInstructionInputTypeFragment(instructionNode, resolvedInputs, renamedArgs, dataArgsManifest, extraArgsManifest, programNode) {
    // Accounts.
    const accountImports = new ImportMap_1.ImportMap();
    const accounts = instructionNode.accounts.map((account) => {
        const typeParam = `TAccount${(0, shared_1.pascalCase)(account.name)}`;
        const resolvedAccount = resolvedInputs.find((input) => input.kind === 'account' && input.name === account.name);
        const optionalSign = !!resolvedAccount.defaultsTo || resolvedAccount.isOptional ? '?' : '';
        const type = getAccountType(resolvedAccount);
        accountImports.mergeWith(type);
        return {
            ...resolvedAccount,
            typeParam,
            optionalSign,
            type: type.render,
        };
    });
    // Arg link imports.
    const argLinkImports = new ImportMap_1.ImportMap();
    if (instructionNode.dataArgs.link) {
        argLinkImports.mergeWith(dataArgsManifest.looseType);
    }
    if (instructionNode.extraArgs.link) {
        argLinkImports.mergeWith(extraArgsManifest.looseType);
    }
    // Arguments.
    const resolveArg = (arg) => {
        var _a, _b;
        const resolvedArg = resolvedInputs.find((input) => input.kind === 'arg' && input.name === arg.name);
        if (((_a = arg.defaultsTo) === null || _a === void 0 ? void 0 : _a.strategy) === 'omitted')
            return [];
        const renamedName = (_b = renamedArgs.get(arg.name)) !== null && _b !== void 0 ? _b : arg.name;
        const optionalSign = arg.defaultsTo || resolvedArg ? '?' : '';
        return [
            {
                ...arg,
                ...resolvedArg,
                renamedName,
                optionalSign,
            },
        ];
    };
    const dataArgsType = instructionNode.dataArgs.link
        ? `${(0, shared_1.pascalCase)(instructionNode.dataArgs.link.name)}Args`
        : `${(0, shared_1.pascalCase)(instructionNode.dataArgs.name)}Args`;
    const dataArgs = instructionNode.dataArgs.link
        ? []
        : instructionNode.dataArgs.struct.fields.flatMap(resolveArg);
    const extraArgsType = instructionNode.extraArgs.link
        ? `${(0, shared_1.pascalCase)(instructionNode.extraArgs.link.name)}Args`
        : `${(0, shared_1.pascalCase)(instructionNode.extraArgs.name)}Args`;
    const extraArgs = instructionNode.extraArgs.link
        ? []
        : instructionNode.extraArgs.struct.fields.flatMap(resolveArg);
    return (0, common_1.fragmentFromTemplate)('instructionInputType.njk', {
        instruction: instructionNode,
        program: programNode,
        accounts,
        dataArgs,
        dataArgsType,
        extraArgs,
        extraArgsType,
    })
        .mergeImportsWith(accountImports, argLinkImports)
        .addImports('solanaAddresses', ['Base58EncodedAddress']);
}
exports.getInstructionInputTypeFragment = getInstructionInputTypeFragment;
function getAccountType(account) {
    const typeParam = `TAccount${(0, shared_1.pascalCase)(account.name)}`;
    if (account.isPda && account.isSigner === false) {
        return (0, common_1.fragment)(`ProgramDerivedAddress<${typeParam}>`).addImports('solanaAddresses', ['ProgramDerivedAddress']);
    }
    if (account.isPda && account.isSigner === 'either') {
        return (0, common_1.fragment)(`ProgramDerivedAddress<${typeParam}> | Signer<${typeParam}>`)
            .addImports('solanaAddresses', ['ProgramDerivedAddress'])
            .addImports('shared', ['Signer']);
    }
    if (account.isSigner === 'either') {
        return (0, common_1.fragment)(`Base58EncodedAddress<${typeParam}> | Signer<${typeParam}>`)
            .addImports('solanaAddresses', ['Base58EncodedAddress'])
            .addImports('shared', ['Signer']);
    }
    if (account.isSigner) {
        return (0, common_1.fragment)(`Signer<${typeParam}>`).addImports('shared', ['Signer']);
    }
    return (0, common_1.fragment)(`Base58EncodedAddress<${typeParam}>`).addImports('solanaAddresses', ['Base58EncodedAddress']);
}
//# sourceMappingURL=instructionInputType.js.map