"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValueNodeFragment = void 0;
const shared_1 = require("../../../shared");
const common_1 = require("./common");
function getValueNodeFragment(value) {
    var _a;
    switch (value.kind) {
        case 'list':
        case 'tuple':
            return (0, common_1.mergeFragments)(value.values.map((v) => getValueNodeFragment(v)), (renders) => `[${renders.join(', ')}]`);
        case 'set':
            return (0, common_1.mergeFragments)(value.values.map((v) => getValueNodeFragment(v)), (renders) => `new Set([${renders.join(', ')}])`);
        case 'map':
            const entryFragments = value.values.map(([k, v]) => (0, common_1.mergeFragments)([getValueNodeFragment(k), getValueNodeFragment(v)], (renders) => `[${renders.join(', ')}]`));
            return (0, common_1.mergeFragments)(entryFragments, (renders) => `new Map([${renders.join(', ')}])`);
        case 'struct':
            const fieldFragments = Object.entries(value.values).map(([k, v]) => getValueNodeFragment(v).mapRender((r) => `${k}: ${r}`));
            return (0, common_1.mergeFragments)(fieldFragments, (renders) => `{ ${renders.join(', ')} }`);
        case 'enum':
            const enumName = (0, shared_1.pascalCase)(value.enumType);
            const enumFn = (0, shared_1.camelCase)(value.enumType);
            const variantName = (0, shared_1.pascalCase)(value.variant);
            const rawImportFrom = (_a = value.importFrom) !== null && _a !== void 0 ? _a : 'generated';
            const importFrom = rawImportFrom === 'generated' ? 'generatedTypes' : rawImportFrom;
            if (value.value === 'scalar') {
                return (0, common_1.fragment)(`${enumName}.${variantName}`).addImports(importFrom, enumName);
            }
            if (value.value === 'empty') {
                return (0, common_1.fragment)(`${enumFn}('${variantName}')`).addImports(importFrom, enumFn);
            }
            return getValueNodeFragment(value.value)
                .mapRender((r) => `${enumFn}('${variantName}', ${r})`)
                .addImports(importFrom, enumFn);
        case 'optionSome':
            return getValueNodeFragment(value.value)
                .mapRender((r) => `some(${r})`)
                .addImports('solanaOptions', 'some');
        case 'optionNone':
            return (0, common_1.fragment)('none()').addImports('solanaOptions', 'none');
        case 'publicKey':
            return (0, common_1.fragment)(`address("${value.value}")`).addImports('solanaAddresses', 'address');
        case 'string':
        case 'number':
        case 'boolean':
            return (0, common_1.fragment)(JSON.stringify(value.value));
        default:
            const neverDefault = value;
            throw new Error(`Unexpected value type ${neverDefault.kind}`);
    }
}
exports.getValueNodeFragment = getValueNodeFragment;
//# sourceMappingURL=valueNode.js.map